@@ -136,6 +136,9 @@ public:
 private:
 	void setProperty(char *dp, char *tp, size_t size);
 
+	void getProperty(char *dp, char *tp, size_t size)
+		{setProperty(tp, dp, size);};
+
 	size_t getPropertySize(void)
 		{return 10;};
 } pWeekday;
