@@ -276,6 +276,18 @@ bool ScriptInterp::scrDisable(void)
 	return true;
 }
 
+void ScriptInterp::rewindTemp(void)
+{
+	tempidx = 0;
+}
+
+void ScriptInterp::setTemp(const char *value)
+{
+	snprintf(temps[tempidx++], getSymbolSize() + 1, "%s", value);
+	if(tempidx >= SCRIPT_TEMP_SPACE)
+		tempidx = 0;
+}
+
 void ScriptInterp::advance(void)
 {
 	script[stack].line = script[stack].line->next;
@@ -340,7 +352,7 @@ retry:
 bool ScriptInterp::step(const char *trapname)
 {
 	unsigned long mask, cmask;
-	bool rtn, sync;
+	bool rtn, sync, skip = false;
 	Symbol *sym = NULL;
 
 	if(!image)
@@ -375,7 +387,10 @@ retry:
 			break;
 		pull();
 		if(script[stack].line)
+		{
 			advance();
+			skip = true;
+		}
 	}
 
 	if(!script[stack].line)
@@ -389,13 +404,20 @@ retry:
 	{
 		if((cmask & script[stack].script->mask) != cmask)
 		{
+			skip = true;
 			advance();
 			goto retry;
 		}
 	}
 
 	if(mask & script[stack].line->mask)
+	{
 		trap(trapname);
+		return true;
+	}
+
+	if(script[stack].line->prescan && skip)
+		return true;
 
 	if(script[stack].line->error)
 		sym = getEntry("script.error");
@@ -508,7 +530,12 @@ bool ScriptInterp::scrSkip(void)
 
 bool ScriptInterp::scrData(void)
 {
-	advance();
+	while(script[stack].line->method == &ScriptInterp::scrData)
+	{
+		advance();
+		if(!script[stack].line)
+			return true;
+	}
 	return true;
 }
 
@@ -661,15 +688,15 @@ bool ScriptInterp::scrSelect(void)
 		case SEL_SUFFIX:
 			if(len < l2)
 				break;
-			if(!stricmp(match, value + len - l2))
+			if(!mapicmp(match, value + len - l2))
 				return scrGoto();
 			break;
 		case SEL_PREFIX:
-			if(!strnicmp(match + offset, value, l2))
+			if(!mapnicmp(match + offset, value, l2))
 				return scrGoto();
 			break;
 		case SEL_NORMAL:
-			if(!stricmp(match, value))
+			if(!mapicmp(match, value))
 				return scrGoto();
 			break;
 		case SEL_LENGTH:
@@ -896,6 +923,114 @@ bool ScriptInterp::scrSwap(void)
 	return true;
 }
 
+bool ScriptInterp::scrMap(void)
+{
+	Line *line;
+	Name *scr;
+	char *cp;
+	const char *prefix = getMember();
+	char *value = getKeyword("table");
+	char namebuf[256];
+	unsigned len;
+	bool find = false;
+
+	enum
+	{
+		MAP_PREFIX,
+		MAP_SUFFIX,
+		MAP_ABSOLUTE,
+		MAP_VALUE
+	}	mapmode = MAP_ABSOLUTE;
+
+	if(prefix)
+        {
+                if(!strnicmp(prefix, "pre", 3))
+                        mapmode = MAP_PREFIX;
+                else if(!strnicmp(prefix, "suf", 3))
+                        mapmode = MAP_SUFFIX;
+		else if(!strnicmp(prefix, "end", 3))
+			mapmode = MAP_SUFFIX;
+		else if(!strnicmp(prefix, "val", 3))
+			mapmode = MAP_VALUE;
+                else if(!strnicmp(prefix, "abs", 3))
+                        mapmode = MAP_ABSOLUTE;
+        }
+
+	if(!value)
+		value = script[stack].script->name;
+	else if(!strnicmp(value, "::", 2))
+        {
+                strcpy(namebuf, script[stack].script->name);
+                strcat(namebuf, value);
+                value = namebuf;
+        }
+
+        scr = getScriptImage(value);
+        if(!scr)
+        {
+                error("no-source-to-read");
+                return true;
+        }
+	line = scr->first;
+
+	value = getValue("*");
+	len = strlen(value);
+	
+	while(line)
+	{
+		if(line->method != &ScriptInterp::scrData)
+		{
+			line = line->next;
+			continue;
+		}	
+		cp = strchr(line->cmd, '.');
+		if(!cp)
+		{
+			line = line->next;
+			continue;
+		}
+		++cp;
+
+		switch(mapmode)
+		{
+		case MAP_VALUE:
+			if(atol(cp) == atol(value))
+				find = true;
+			break;
+		case MAP_ABSOLUTE:
+			if(!mapicmp(cp, value))
+				find = true;
+			break;
+		case MAP_PREFIX:
+			if(!mapnicmp(cp, value, len))
+				find = true;
+			break;
+		case MAP_SUFFIX:
+			if(strlen(cp) > len)
+				break;
+
+			cp = cp + strlen(cp) - len;
+			if(!mapicmp(cp, value))
+				find = true;
+			break;
+		}
+						
+		if(find)
+			break;
+
+		line = line->next;
+	}
+
+	if(line)
+	{
+		setLine(line);
+		return scrGoto();		
+	}
+
+	error("no-map-data");
+	return true;
+}	
+
 bool ScriptInterp::scrRead(void)
 {
 	Symbol *sym;
@@ -910,7 +1045,10 @@ bool ScriptInterp::scrRead(void)
 		mem = "";
 	if(!stricmp(mem, "from"))
 	{
-		value = getValue(script[stack].script->name);
+		value = getKeyword("table");
+		if(!value)
+			value = getValue(script[stack].script->name);
+
 		if(!strnicmp(value, "::", 2))
 		{
 			strcpy(namebuf, script[stack].script->name);
@@ -3232,6 +3370,12 @@ bool ScriptInterp::getOnce(void)
 	return ret;
 }
 
+void ScriptInterp::setLine(Line *line)
+{
+	script[stack].line = line;
+	script[stack].index = 0;
+}
+
 bool ScriptInterp::redirect(const char *scriptname)
 {
 	Name *scr;
@@ -3452,6 +3596,14 @@ char *ScriptInterp::getContent(char *opt)
 	if(!opt)
 		return NULL;
 
+	if(*opt == '!' && script[stack].line->prescan)
+	{
+		opt = temps[tempidx++];
+		if(tempidx >= SCRIPT_TEMP_SPACE)
+			tempidx = 0;
+		return opt;
+	}
+
 	if(*opt != '%' && *opt != '@')
 		return opt;
 
@@ -3605,6 +3757,17 @@ char *ScriptInterp::getValue(const char *def)
 	return opt;
 }	
 
+Script::Line *ScriptInterp::getPrescan(void)
+{
+	if(!script[stack].line)
+		return NULL;
+
+	if(script[stack].line->prescan)
+		return script[stack].line;
+
+	return NULL;
+}
+
 const char *ScriptInterp::getMember(void)
 {
 	char *cmd = script[stack].line->cmd;
