@@ -89,7 +89,7 @@ Keydata(cfgfile), Mutex()
 		{"if", &ScriptInterp::scrIf, &ScriptCommand::chkHasArgs},
 		{"do", &ScriptInterp::scrDo, &ScriptCommand::chkIgnore},
 		{"for", &ScriptInterp::scrFor, &ScriptCommand::chkHasList},
-		{"read", &ScriptInterp::scrRead, &ScriptCommand::chkHasArgs},
+		{"read", &ScriptInterp::scrRead, &ScriptCommand::chkIgnore},
 		{"tryeach", &ScriptInterp::scrTryeach, &ScriptCommand::chkHasArgs},
 		{"foreach", &ScriptInterp::scrForeach, &ScriptCommand::chkHasArgs},
 		{"repeat", &ScriptInterp::scrRepeat, &ScriptCommand::chkHasArgs},
@@ -140,6 +140,7 @@ Keydata(cfgfile), Mutex()
 		{"otherwise", &ScriptInterp::scrCase, &ScriptCommand::chkNoArgs},
 		{"label", &ScriptInterp::scrLabel, &ScriptCommand::chkHasArgs},
 		{"ref", &ScriptInterp::scrRef, &ScriptCommand::chkHasArgs},
+		{"map", &ScriptInterp::scrMap, &ScriptCommand::chkHasArgs},
 		{NULL, NULL, NULL}};
 
 	int i;
@@ -1156,13 +1157,20 @@ void ScriptImage::commit(void)
 	cmds->leaveMutex();
 }
 
-char *ScriptImage::getToken(void)
+char *ScriptImage::getToken(char **pre)
 {
 	static char temp[513];
 	char *cp = temp + 1;
 	char *base = temp + 1;
 	char q;
 
+	if(pre)
+		if(*pre)
+		{
+			cp = *pre;
+			*pre = NULL;
+		}
+
 	if(*bp == '=')
 	{
 		++bp;
@@ -1454,7 +1462,7 @@ int ScriptImage::compile(istream *str, char *name, const char *scrname)
 {
 	const char *basename = name;
 	char namebuf[256];
-	char *command, *token;
+	char *command, *token, *pretoken;
 	char *args[33];
 	char *err, *cp;
 	bool trapflag, errflag, syncflag;
@@ -1476,6 +1484,13 @@ int ScriptImage::compile(istream *str, char *name, const char *scrname)
 
 	scrStream = str;
 
+	enum 
+	{
+		cSCRIPT,
+		cDATA,
+		cMAP
+	}	cmode = cSCRIPT;
+
 compile:
 	trapflag = false;
 	count = number = 0;
@@ -1512,7 +1527,7 @@ compile:
 		if(!then)
 		{
 			cmask = 0;
-			errflag = syncflag = false;
+			errflag = syncflag;
 			scrStream->getline(buffer + offset, sizeof(buffer) - 1 - offset);
 			if(scrStream->eof())
 				break;
@@ -1612,6 +1627,24 @@ compile:
 			continue;
 		}
 
+		if(!stricmp(token, ".script") || !stricmp(token, ".code"))
+		{
+			cmode = cSCRIPT;
+			continue;
+		}
+
+		if(!stricmp(token, ".map"))
+		{
+			cmode = cMAP;
+			continue;
+		}
+
+		if(!stricmp(token, ".data"))
+		{
+			cmode = cDATA;
+			continue;
+		}
+
 		if(!stricmp(token, ".use"))
 		{
 			while(NULL != (token = getToken()))
@@ -1749,6 +1782,25 @@ retoken:
 		if(!token)
 			continue;
 
+
+		switch(cmode)
+		{
+		case cSCRIPT:
+			pretoken = NULL;
+			break;
+		case cDATA:
+			pretoken = token;
+			token = "data";
+			break;
+		case cMAP:
+			pretoken = (char *)alloc(strlen(token) + 6);
+			strcpy(pretoken, "data.");
+			strcat(pretoken, token);
+			token = pretoken;
+			pretoken = NULL;
+		}
+
+		
 		if(!stricmp(token, "optional") || !stricmp(token, "unsafe"))
 		{
 			ignore = true;
@@ -1794,7 +1846,7 @@ retoken:
 		command = alloc(token);
 	
 		argc = 0;
-		while(argc < 32 && NULL != (token = getToken()))
+		while(argc < 32 && NULL != (token = getToken(&pretoken)))
 		{
 			if(token[0] == '$' && isalpha(token[1]))
 			{
@@ -1828,6 +1880,7 @@ retoken:
 		line->argc = --argc;
 		line->error = errflag;
 		line->sync = syncflag;
+		line->prescan = false;
 		line->method = handler;
 		line->cmd = command;
 		line->loop = 0;
