@@ -69,6 +69,7 @@ class CCXX_CLASS_EXPORT ScriptInterp;
 #define SCRIPT_INDEX_SIZE KEYWORD_INDEX_SIZE
 #define	SCRIPT_PREPROCESSOR_OVERRIDE 1
 #define	SCRIPT_EXCLUSIVE_OVERRIDE 1
+#define	SCRIPT_MAP_TABLES 1
 
 class Script
 {
@@ -122,6 +123,7 @@ protected:
 		unsigned char argc;
 		bool error : 1;
 		bool sync : 1;
+		bool prescan : 1;
 		Method method;
 		char *cmd;
 		char **args;
@@ -585,6 +587,13 @@ protected:
 	 * @param keydata entry for constants.
 	 */
 	ScriptCommand(const char *cfgfile);
+
+public:
+	virtual	int mapicmp(const char *s1, const char *s2)
+		{return stricmp(s1, s2);};
+
+	virtual int mapnicmp(const char *s1, const char *s2, size_t n)
+		{return strnicmp(s1, s2, n);};
 };
 
 /**
@@ -847,7 +856,7 @@ protected:
 	friend class ScriptInterp;
 	friend class ScriptModule;
 
-	char *getToken(void);
+	char *getToken(char **pre = NULL);
 
 	/**
 	 * Get the interpreter method pointer for a given keyword.
@@ -1067,6 +1076,7 @@ private:
 	bool scrEndif(void);
 	bool scrFor(void);
 	bool scrRead(void);
+	bool scrMap(void);
 	bool scrRepeat(void);
 	bool scrForeach(void);
 	bool scrTryeach(void);
@@ -1134,6 +1144,13 @@ protected:
 	 */
 	unsigned long getMask(void);
 
+	/**
+	 * Set the line handler to a new statement.
+	 *
+	 * @param line value.
+	 */
+	void setLine(Line *line);
+
 	/**
 	 * Fetch the script mask identifer.
 	 *
@@ -1204,6 +1221,18 @@ public:
 	 */
 	Symbol *getVariable(size_t size = 0);
 
+	/**
+	 * Rewind temporary buffer for prescan results.
+	 */
+	void rewindTemp(void);
+
+	/**
+	 * Set prescan value into temporary buffer.
+	 *
+	 * @param value to set into temp.
+	 */
+	void setTemp(const char *value);
+
 
 protected:
 	/**
@@ -1450,6 +1479,13 @@ public:
 	inline Line *getScript(void)
 		{return script[stack].line;};
 
+	/**
+	 * Return if the current line requires prescanning.
+	 *
+	 * @return line object if prescanned line else NULL.
+	 */
+	Line  *getPrescan(void);
+
 	/**
 	 * Return the member id of a script command.
 	 *
@@ -1481,6 +1517,12 @@ public:
 	 */
 	inline void autoloop(bool enable)
 		{loop = enable;};
+
+	inline int mapicmp(const char *s1, const char *s2)
+		{return cmd->mapicmp(s1, s2);};
+
+	inline int mapnicmp(const char *s1, const char *s2, size_t n)
+		{return cmd->mapnicmp(s1, s2, n);};
 };
 
 #ifdef	CCXX_NAMESPACES
